/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include "common.h"
#include "../include/srfd_interface.h"
#include "../include/srfd_label.h"



void srfd_display_payload(struct srfd_payload * payload)
{

	switch (payload->cmd) {

	case SRFD_GET_SUBJECT_LABEL:
		printf("pid=%d\n",payload->fd);	
		printf("return error code=%d\n",payload->error);	
		printf("cmd=%d\n",payload->cmd);	

		printf("** Subject Label Properties **\n");
		printf("output_invul = %d\n", SRFD_label2int(payload->file_lbl.subj_lbl.output_invul));
		printf("input_invul = %d\n", SRFD_label2int(payload->file_lbl.subj_lbl.input_invul));
		printf("invul = %d\n", SRFD_label2int(payload->file_lbl.subj_lbl.invul_lbl));
		printf("min = %d\n", SRFD_label2int(payload->file_lbl.subj_lbl.min_lbl));
		printf("current = %d\n", SRFD_label2int(payload->file_lbl.subj_lbl.current_lbl));
		printf("can_handle_write_errors = %d\n", payload->file_lbl.subj_lbl.can_handle_write_errors);
		printf("can_handle_read_errors = %d\n", payload->file_lbl.subj_lbl.can_handle_write_errors);
		break;

	case SRFD_SET_LABEL:
	case SRFD_GET_LABEL:
		printf("name=%s\n",payload->user_name);	
		printf("isfd=%d\n",payload->isfd);	
		printf("return error code=%d\n",payload->error);	
		printf("cmd=%d\n",payload->cmd);	

		printf("** Object Label Properties **\n");
		printf("current_min = %d\n",	SRFD_label2int(payload->file_lbl.obj_lbl.current_min_lbl));
		printf("min = %d\n", 		SRFD_label2int(payload->file_lbl.obj_lbl.min_lbl));
		printf("current = %d\n", 	SRFD_label2int(payload->file_lbl.obj_lbl.current_lbl));

		printf("** Subject Label Properties **\n");
		printf("output_invul = %d\n", 	SRFD_label2int(payload->file_lbl.subj_lbl.output_invul));
		printf("input_invul = %d\n", 	SRFD_label2int(payload->file_lbl.subj_lbl.input_invul));
		printf("invul = %d\n", 		SRFD_label2int(payload->file_lbl.subj_lbl.invul_lbl));
		printf("min = %d\n", 		SRFD_label2int(payload->file_lbl.subj_lbl.min_lbl));
		printf("current = %d\n", 	SRFD_label2int(payload->file_lbl.subj_lbl.current_lbl));
		printf("can_handle_write_errors = %d\n", payload->file_lbl.subj_lbl.can_handle_write_errors);
		printf("can_handle_read_errors = %d\n", payload->file_lbl.subj_lbl.can_handle_write_errors);
		break;
	default:
		printf("Unsupported cmd %d\n", payload->cmd);

	}

}


