/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../include/srfd_interface.h"
#include "common.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <errno.h>

int main(int argc, char *argv[])
{
    
	int ret = 0;
	struct srfd_payload payload;
	char input_file[PATH_MAX];
	int myfd = 0;

    	if (argc < 2) {
        	printf("too few args\n");
        	printf("getlabel <filename> \n");
        	exit(0);
    	}
    
    	if (argc > 2) {
        	printf("too many args\n");
        	printf("getlabel <filename> \n");
        	exit(0);
    	}
	
	strcpy(input_file, argv[1]);
	payload.user_name = (char *)&input_file;
#if 0
	printf("name = %s \n", payload.user_name);
	myfd = open(payload.user_name,O_RDWR);
	if(myfd < 0){
		perror("fd problem");
	}
#endif
	payload.isfd = 0;
	payload.fd = myfd;
	payload.error = -1;
	payload.cmd = SRFD_GET_LABEL;

	ret = srfd_call_interface(&payload);
	if (ret < 0) {
		if (errno == ENOENT)
			printf("SRFD Module is not loaded!\n");
		return -1;
	}
    	srfd_display_payload(&payload);
    	return 0;
    
}
