/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../include/srfd_interface.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>
#include "../include/srfd_label.h"
#include "common.h"
#include <errno.h>


int main(int argc, char *argv[])
{
    
	int ret = 0;
	struct srfd_payload payload;
	char input_file[80];
	int max = ((pow(2,(int)MAX_LBL_BITS))-1);

	memset(&payload, 0, sizeof(struct srfd_payload));


	if (geteuid() != 0) {
		printf("Must run as root\n");
		exit(-1);
	}


    	if (argc < 4) {
        	printf("too few args\n");
        	printf("setlabel <filename> <current_label> <min_label>\n");
        	exit(0);
    	}
    
    	if (argc > 4) {
        	printf("too many args\n");
        	printf("setlabel <filename> <current_label> <min_label>\n");
        	exit(0);
    	}
	
	strcpy(input_file, argv[1]);
	payload.user_name = (char *)&input_file;
	printf("name = %s \n", payload.user_name);
	payload.isfd = 0;
	payload.fd = 0;
	payload.error = -1;
/*	if(((atoi(argv[2]) != 0) && (atoi(argv[2]) != 1)) ||
			((atoi(argv[3]) != 1) && (atoi(argv[3]) != 0))) {
		printf("INVALID input: read_log / write_log can be 0 or 1\n");
		exit(0);
	}
*/	if((atoi(argv[2]) < 0) || (atoi(argv[2]) > max) ||
			(atoi(argv[3]) < 0) || (atoi(argv[3]) > max)) {
		printf("INVALID input: label values can be between 0 and %d\n",max);
		exit(0);
	}
	payload.file_lbl.obj_lbl.current_lbl.level = (unsigned char)atoi(argv[2]);
	payload.file_lbl.obj_lbl.min_lbl.level = (unsigned char)atoi(argv[3]);
	payload.file_lbl.obj_lbl.current_min_lbl.level = (unsigned char)atoi(argv[3]);
	payload.cmd = SRFD_SET_LABEL;

	ret = srfd_call_interface(&payload);
	if (ret < 0) {
		printf("interface function returned error: %d\n",ret);
		if (errno == ENOENT)
			printf("SRFD Module is not loaded!\n");
		return -1;
	}

	if (payload.error != 0) {
		printf("ERROR setting Label: %d\n",payload.error);
		exit(0);
	}
	else {
		printf("SUCCESS setting Label: %d\n",payload.error);
		printf("You entered the following :\n\n");
    		srfd_display_payload(&payload);
	}
    	return 0;
    
}
