/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __SRFD_DEBUG_H__
#define __SRFD_DEBUG_H__

#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/security.h>
#include <linux/sched.h>
//#include "srfd_file.h"

#include "../module/srfd_security.h"
#include "../module/srfd_netlink.h"


extern char srfd_debug_buffer[1024];

#define srfd_procIs(_proc) (strcmp(current->comm, _proc) == 0)


#define SRFD_IS_INTERESTING_PROC 0 //(srfd_procIs("cat")||srfd_procIs("sort")||srfd_procIs("bash")||srfd_procIs("vi"))


#define srfd_log(format, args...) \
	do { \
		if (!srfd_procIs("dmesg")) \
		printk(KERN_INFO "%s (%d) %s (%d) " format "\n", current->comm, current->pid, __FUNCTION__,__LINE__, ##args); \
/*		struct file *file = srfd_file_open("/tmp/log", O_WRONLY|O_APPEND, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH); \
		int len; \
		if (file) { \
			len = snprintf(srfd_debug_buffer, 1024, format "\n", ##args); \
			srfd_file_write(file, srfd_debug_buffer, len, 0); \
			srfd_file_close(file); \
		} \
*/	} while (0)


#define srfd_alwayslog(format, args...) srfd_log(format, ##args);

#define srfd_logViolation(format, args...) \
	do { \
		srfd_alwayslog("[SRFD_VIOLATION] " format, ##args); \
		srfd_nl_message(srfd_log, format, ##args); \
	} while (0)


#define srfd_logDowngrade(format, args...) \
	do { \
		srfd_alwayslog("[SRFD_DOWNGRADE] " format, ##args); \
		srfd_nl_message(srfd_downgrade, format, ##args); \
	} while (0)

#define srfd_logTEMP(format, args...) \
	do { \
		srfd_alwayslog("[SRFD_TEMP] " format, ##args); \
/*		srfd_nl_message(srfd_temp, format, ##args); \
*/	} while (0)



#define srfd_info(format, args...)
//#define SRFD_VERBOSE_LOGGING
//#define srfd_info(format, args...) srfd_log(format, ##args)

#define srfd_error(format, args...) srfd_alwayslog("[SRFD_ERROR] " format, ##args)

#define srfd_assert(expr, format, args...) \
        do { \
                if (!(expr))    \
                        srfd_alwayslog("Assert failed (%s) "format, #expr, ##args); \
        } while (0)

#define srfd_assert1(expr) \
        do { \
                if (!(expr))    \
                        srfd_alwayslog("Assert failed (%s) ", #expr); \
        } while (0)

#define srfd_assert1r(expr) \
	({ 	int _value = (expr); \
		if (!_value) \
                        srfd_alwayslog("Assert failed (%s) ", #expr); \
	(!(_value)); })

#define srfd_assertr(expr, format, args...) \
	({ 	int _value = (expr); \
		if (!_value) \
                        srfd_alwayslog("Assert failed (%s) "format, #expr, ##args); \
	(!(_value)); })






#endif /* __SRFD_DEBUG_H__ */
