/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

/* 
 * This interface file contains the interface ioctl definition of the SRFD
 * preservation module. Currently the ioctls exposed below are the only way
 * for the user to change the label of objects through the SRFD module.
 */
#ifndef SRFD_IOCTL_H
#define SRFD_IOCTL_H

#include <linux/ioctl.h>
#include "srfd_label.h"


#if __KERNEL__
#include <linux/dcache.h>

extern int srfd_get_security_label_for_existing_inode(
		struct dentry *fil_dentry);
extern int srfd_get_disk_security_label(struct dentry *fil_dentry,
		struct SRFD_file_lbl *file_lbl);
extern int srfd_set_security_label(struct dentry *fil_dentry, 
	struct SRFD_object_lbl *object_lbl, struct SRFD_subject_lbl *subject_lbl);


#else
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#endif

#define SRFD_INTERFACE_WRITE_FILE "/sys/kernel/security/srfd/srfd_interface_set"
#define SRFD_INTERFACE_READ_FILE "/sys/kernel/security/srfd/srfd_interface_get"

typedef enum SRFD_CMD {
	
	SRFD_SET_LABEL,
	SRFD_GET_LABEL,
	SRFD_SET_LEVEL,
	SRFD_GET_LEVEL,
	SRFD_GET_SUBJECT_LABEL,
	SRFD_DUMP_GRAPH

}srfdcmd_t;

struct srfd_payload {
	char * user_name;
	int isfd;
	struct SRFD_file_lbl file_lbl; 
	int fd;
	int error;
	srfdcmd_t cmd;
	unsigned int flag;
};

extern int srfd_create_srfdfs(void);
extern void srfd_destroy_srfdfs(void);

#define SRFD_INTERFACEREAD	1
#define SRFD_INTERFACEWRITE	2
#define SRFD_INTERFACEREADWRITE	3

#if __KERNEL__
#else
static int srfd_call_interface (struct srfd_payload *payload)
{
	int error = 0;
	int fd;	

	fd = open(SRFD_INTERFACE_WRITE_FILE, O_WRONLY);
	if (fd < 0) {
		perror("interface write file - open failure");
		return fd;
	}
	error = write(fd, (void *)payload, sizeof(struct srfd_payload));
	if (error < 0) {
		printf("fd: %d, pointer: %p, size: %d\n", fd, payload, sizeof(struct srfd_payload));
		perror("interface write file - write failure");
		close(fd);
		return error;
	}
	close(fd);

	fd = open(SRFD_INTERFACE_READ_FILE, O_RDONLY);
	if (fd < 0) {
		perror("interface read file - open failure");
		return fd;
	}
	error = read(fd, (void *)payload, sizeof(struct srfd_payload));
	if (error < 0) {
		perror("interface read file - read failure");
		close(fd);
		return error;
	}
	close(fd);
	return error;
}
#endif
#endif
