/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __PPIU_DEBUG_H__
#define __PPIU_DEBUG_H__

//#define _GNU_SOURCE
#include <unistd.h>
#include <sys/syscall.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define PPIU_LOGGING_PATH "/home/ubuntu/Desktop/ppiu_log"
#define ALL_LOGGING_PATH "/home/ubuntu/Desktop/ppiu_all_log"

#define PPI_LOGG(format, args...) PPIU_CUSTOM_LOG(PPIU_LOGGING_PATH, format, ##args) //printf(format "\n", ##args)

#define PPI_OK PPI_LOGG
#define PPI_ERROR PPI_LOGG
#define PPI_NOT_EXPECTED PPI_LOGG
#define PPI_INFO PPI_LOGG

#define PPI_LOG(_level, _format, _args...) PPI_ ##_level( "[" #_level "] " _format, ##_args)
#define PPIU_LOG PPI_LOG

#define MAX_LOG_MSG_LEN 1024

extern __thread char ppiu_debug_msg_buf[MAX_LOG_MSG_LEN];


#define PPIU_CUSTOM_LOG(PATH, format, args...)				\
  do {									\
    int saved_errno = errno;						\
    int byteCount = 0;							\
    int pFile = syscall(SYS_open, PATH, O_WRONLY|O_APPEND, S_IRWXU|S_IWGRP|S_IWOTH); \
    if (pFile == -1 && errno == ENOENT) \
      pFile = syscall(SYS_open, PATH, O_WRONLY|O_APPEND|O_CREAT, S_IRWXU|S_IWGRP|S_IWOTH); \
    if (pFile > 0) {							\
      byteCount = snprintf(ppiu_debug_msg_buf, MAX_LOG_MSG_LEN, format "\n", ##args); \
      syscall(SYS_write, pFile, ppiu_debug_msg_buf, byteCount);		\
      syscall(SYS_close, pFile);					\
      pFile = syscall(SYS_open, ALL_LOGGING_PATH, O_WRONLY|O_APPEND|O_CREAT, S_IRWXU|S_IWGRP|S_IWOTH); \
      if (pFile > 0) {							\
	syscall(SYS_write, pFile, ppiu_debug_msg_buf, byteCount);	\
	syscall(SYS_close, pFile);					\
      }									\
    }									\
    errno = saved_errno;						\
  } while (0)




#endif /* __PPIU_DEBUG_H__ */

