/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "srfd_security.h"
#include "srfd_debug.h"

int srfd_addCredsec2Subgroup(struct SRFD_credsec *credsec, struct SRFD_subgroup *subgroup) {

	if (credsec->cs_sg != NULL) {
		if (credsec->cs_sg == subgroup)
			return 0;

		srfd_alwayslog("oldsubgroup: %p, newsubgroup: %p", credsec->cs_sg, subgroup);
		srfd_removeCredsecFromSubgroup(credsec);
	}

	credsec->cs_sg = srfd_get_subgroup(subgroup);
	atomic_inc(&subgroup->sg_cred_rc);
	return 0;
}


int srfd_removeCredsecFromSubgroup(struct SRFD_credsec *credsec) {

	struct SRFD_handle *handle;
	struct SRFD_subgroup *subgroup;

	if (srfd_assertr(credsec->cs_sg != NULL, "remove credsec %p from null subgroup", credsec))
		goto out;

	subgroup = credsec->cs_sg;
	atomic_dec(&credsec->cs_sg->sg_cred_rc);

	if (atomic_read(&credsec->cs_sg->sg_cred_rc) == 0) {
		if (srfd_assert1r(srfd_subgroup_isValid(subgroup) == 0))
			goto next;

		srfd_do_foreach_subgroup_all_rh(subgroup, handle) {
			srfd_removeHandleFromObject(handle);
			srfd_removeHandleFromSubgroup(handle);
			srfd_put_handle(handle);

		} srfd_while_foreach_subgroup_rh(subgroup);

		srfd_do_foreach_subgroup_all_wh(subgroup, handle) {
			srfd_removeHandleFromObject(handle);
			srfd_removeHandleFromSubgroup(handle);
			srfd_put_handle(handle);
		} srfd_while_foreach_subgroup_wh(subgroup);

	}

next:
	srfd_put_subgroup(credsec->cs_sg);
	credsec->cs_sg = NULL;

out:
	return 0;
}



