/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "srfd_security.h"
#include "srfd_debug.h"

int srfd_addHandle2Object(struct SRFD_handle *handle, struct SRFD_object *object) {
	unsigned long flags;

	handle->h_o = srfd_get_object(object);

	switch (handle->h_type) {
	case SRFD_HANDLE_TYPE_READ:
		spin_lock_irqsave(&object->o_rh_list_lock, flags);
		list_add_rcu(&handle->h_o_list, &object->o_rh_list);
		spin_unlock_irqrestore(&object->o_rh_list_lock, flags);
		break;
	case SRFD_HANDLE_TYPE_WRITE:
		spin_lock_irqsave(&object->o_wh_list_lock, flags);
		list_add_rcu(&handle->h_o_list, &object->o_wh_list);
		spin_unlock_irqrestore(&object->o_wh_list_lock, flags);
		break;
	default:
		srfd_error("Unhandled handle type");
	}

	return 0;
}

void srfd_removeHandleFromObject(struct SRFD_handle *handle) {
	unsigned long flags;

	struct SRFD_object *object = srfd_get_object(handle->h_o);
	if (srfd_assert1r(object != NULL))
		goto out;

	switch (handle->h_type) {
	case SRFD_HANDLE_TYPE_READ:
		spin_lock_irqsave(&object->o_rh_list_lock, flags);
		list_del_rcu(&handle->h_o_list);
		spin_unlock_irqrestore(&object->o_rh_list_lock, flags);
		break;
	case SRFD_HANDLE_TYPE_WRITE:
		spin_lock_irqsave(&object->o_wh_list_lock, flags);
		list_del_rcu(&handle->h_o_list);
		spin_unlock_irqrestore(&object->o_wh_list_lock, flags);
		break;
	default:
		srfd_error("Unhandled handle type");
	}

	srfd_put_object(object);
	srfd_put_object(object);

out:
	return;
}

