/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "srfd_security.h"
#include "srfd_debug.h"

int srfd_addHandle2Subgroup(struct SRFD_handle *handle, struct SRFD_subgroup *subgroup) {

	unsigned long flags;

	switch (handle->h_type) {
	case SRFD_HANDLE_TYPE_READ:
		spin_lock_irqsave(&subgroup->sg_rh_list_lock, flags);
		list_add_rcu(&handle->h_sg_list, &subgroup->sg_rh_list);
		spin_unlock_irqrestore(&subgroup->sg_rh_list_lock, flags);
		break;
	case SRFD_HANDLE_TYPE_WRITE:
		spin_lock_irqsave(&subgroup->sg_wh_list_lock, flags);
		list_add_rcu(&handle->h_sg_list, &subgroup->sg_wh_list);
		spin_unlock_irqrestore(&subgroup->sg_wh_list_lock, flags);
		break;
	default:
		srfd_error("Unhandled handle type");
	}

	handle->h_sg = srfd_get_subgroup(subgroup);
	handle->h_sg = subgroup;

	return 0;
}


int srfd_removeHandleFromSubgroup(struct SRFD_handle *handle) {
	unsigned long flags;

	struct SRFD_subgroup *subgroup = srfd_get_subgroup(handle->h_sg);

	if (srfd_assertr(subgroup != NULL, "handle rc is %d", atomic_read(&handle->h_rc)))
		return 0;

	switch (handle->h_type) {
	case SRFD_HANDLE_TYPE_READ:
		spin_lock_irqsave(&subgroup->sg_rh_list_lock, flags);
		list_del_rcu(&handle->h_sg_list);
		spin_unlock_irqrestore(&subgroup->sg_rh_list_lock, flags);
		break;
	case SRFD_HANDLE_TYPE_WRITE:
		spin_lock_irqsave(&subgroup->sg_wh_list_lock, flags);
		list_del_rcu(&handle->h_sg_list);
		spin_unlock_irqrestore(&subgroup->sg_wh_list_lock, flags);
		break;
	default:
		srfd_error("Unhandled handle type");
	}

	srfd_put_subgroup(subgroup);
	srfd_put_subgroup(subgroup);
	handle->h_sg = NULL;

	return 0;
}


