/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "srfd_security.h"
#include "srfd_internal.h"
#include "srfd_debug.h"


struct file *srfd_fget(struct files_struct *files, int fd)
{
        struct file *file;

	if (srfd_assert1r(files != NULL))
		return NULL;


	srfd_assert1(files != NULL);


        rcu_read_lock();
        file = fcheck_files(files, (unsigned int)fd);
        if (file) {
		srfd_assert1(file != NULL);
                if (!atomic_inc_not_zero(&file->f_count)) {
                        rcu_read_unlock();
                        return NULL;
                }
        }
        rcu_read_unlock();
        return file;
}




char *srfd_convertDentry2FullPath(struct dentry *dentry, char **buffer) {

	char *ret = NULL;
	char *resultPath = NULL;

	*buffer = (char *)__get_free_page(GFP_KERNEL);

	if (!IS_ERR(*buffer)) {

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
		resultPath = dentry_path_raw(dentry, *buffer, PAGE_SIZE);
#else
		if (dentry && dentry->d_op && dentry->d_op->d_dname)
			resultPath = dentry->d_op->d_dname(dentry, *buffer, PAGE_SIZE);
#endif
		if (IS_ERR(resultPath) || resultPath == NULL)
			srfd_freeDentry2FullPath(*buffer);
		else
			ret = resultPath;
	}
	return ret;
}


char *srfd_convertInode2FullPath(struct inode *inode, char **buffer) {

	struct dentry *d;
	char *ret = NULL;


	d = d_find_alias(inode);
	if (d) {
		ret = srfd_convertDentry2FullPath(d, buffer);
		dput(d);
	}	
	return ret;
}









