/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __SRFD_INTERNEL_H__
#define __SRFD_INTERNEL_H__

#include <linux/fdtable.h>
#include <linux/file.h>
#include <linux/mm.h>
#include "security_ops_finder.h"
#include "srfd_security.h"

/* Helper functions */


extern srfd_put_files_struct srfd_exported_put_files_struct;
extern srfd_get_files_struct srfd_exported_get_files_struct;


struct file *srfd_fget(struct files_struct *files, int fd);
#define srfd_fput(_file) fput(_file)

char *srfd_convertInode2FullPath(struct inode *inode, char **fullPath);
char *srfd_convertDentry2FullPath(struct dentry *dentry, char **buffer);
#define srfd_freeInode2FullPath(_buf) free_page((long unsigned int)(_buf))
#define srfd_freeDentry2FullPath srfd_freeInode2FullPath


#define srfd_printObjectInfo2(_printMethod, _object) srfd_printObjectInfo4(_printMethod, _object, "")

#define srfd_printObjectInfo4(_printMethod, _object, _format, args...) do { \
		char *fullPath, *buffer; \
		fullPath = srfd_convertInode2FullPath(_object->o_inode, &buffer); \
		_printMethod("\t" _format " Object Info: ino %lu, Path %s, cur level: %d cur min level: %d, min level: %d", ##args, _object->o_ino, fullPath, srfd_get_objectLabelV(_object, current_lbl), srfd_get_objectLabelV(_object, current_min_lbl), srfd_get_objectLabelV(_object, min_lbl)); \
		if (fullPath) srfd_freeInode2FullPath(buffer); \
	} while (0)


#define srfd_printSubgroupInfo2(_printMethod, _subgroup) srfd_printSubgroupInfo4(_printMethod, _subgroup, "")

#define srfd_printSubgroupInfo4(_printMethod, _subgroup, _format, args...) do { \
		struct SRFD_subgroup *__subgroup = _subgroup; /*For some reason, must cache it, ow. it can be NULL for no reason!*/ \
		if (srfd_get_subgroup(__subgroup)) { \
			char *processesList; \
			processesList = srfd_subgroup_getProcessesList(__subgroup); \
			_printMethod("\t" _format " Subgroup %p has processes: %s, cur level: %d, cur min level: %d, min level: %d", ##args, __subgroup, processesList, \
			srfd_get_subgroupLabelV(__subgroup, current_lbl), srfd_get_subgroupLabelV(__subgroup, current_min_lbl), srfd_get_subgroupLabelV(__subgroup, min_lbl)); \
			if (processesList) { \
				srfd_subgroup_freeProcessesList(processesList); \
			} \
			srfd_put_subgroup(__subgroup); \
		} \
	} while (0)

/*DO NOT USE *_remove_invalid If THESE ARE USED IN THE PROCESS OF MIDDLE OF OPEN */
#define srfd_printSubgroupWriteObjects(_printMethod, _subgroup) do {\
		struct SRFD_handle *handle, *handle2; \
		srfd_do_foreach_subgroup_valid_wh(_subgroup, handle) { \
			srfd_printObjectInfo4(_printMethod, handle->h_o, "subgroup %p Write to ", _subgroup); \
			srfd_do_foreach_object_all_rh(handle->h_o, handle2) { \
				srfd_printSubgroupInfo4(_printMethod, handle2->h_sg, "\t Read by "); \
			} srfd_while_foreach_object_rh(handle->h_o); \
		} srfd_while_foreach_subgroup_rh(_subgroup); \
	} while (0)


#define srfd_printSubgroupReadObjects(_printMethod, _subgroup) do {\
		struct SRFD_handle *handle, *handle2; \
		srfd_do_foreach_subgroup_valid_rh(_subgroup, handle) { \
			srfd_printObjectInfo4(_printMethod, handle->h_o, "subgroup %p Read from ", _subgroup); \
			srfd_do_foreach_object_all_wh(handle->h_o, handle2) { \
				srfd_printSubgroupInfo4(_printMethod, handle2->h_sg, "\t Written by "); \
			} srfd_while_foreach_object_rh(handle->h_o); \
		} srfd_while_foreach_subgroup_rh(_subgroup); \
	} while (0)



#endif /* __SRFD_INTERNEL_H__ */
