/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include <linux/module.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include "srfd_security.h"
#include "srfd_interface.h"
#include "srfd_debug.h"
#include "security_ops_finder.h"
#include "srfd_lsm.h"

#include "srfd_netlink.h"

struct security_operations *security_ops = NULL;
int phase = 1;

static int __init srfd_init(void) {

	struct task_struct *q, *p;
        struct SRFD_credsec *credsec;
	struct SRFD_subgroup *subgroup;
	struct SRFD_subpid *subpid;

	srfd_security_init();
	srfd_create_srfdfs();


	rcu_read_lock();

        for_each_process(p) {
                subgroup = srfd_subgroup_alloc();

                q = p;
                do {
			if (srfd_credsec_findByCred(p->cred) == NULL) {
				credsec = srfd_credsec_allocAndGet(p->cred);
				if (srfd_assert1r(credsec != NULL))
					continue;
				srfd_addCredsec2Subgroup(credsec, subgroup);
			}
			subpid = srfd_subpid_allocAndGet(task_pid(p));
			srfd_addSubpid2Subgroup(subpid, subgroup);
                } while ((p = next_thread(p)) != q);
        }

	rcu_read_unlock();


	security_ops = ccs_find_security_ops2();
	srfd_update_security_ops(security_ops);
	srfd_alwayslog("SRFD module initialized");

	srfd_printStatistics();

	srfd_nl_init();

	return 0;
}

static void __exit srfd_exit (void) {
	


	srfd_restore_security_ops(security_ops);

	srfd_destroy_srfdfs();
	srfd_security_fin();


	srfd_nl_fin();
	srfd_alwayslog("SRFD exit");
	
}



security_initcall (srfd_init);
module_exit (srfd_exit);

MODULE_DESCRIPTION("SRFD LSM module");
MODULE_LICENSE("GPL");


