/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __SRFD_SECURITY_INTERNAL_H__
#define __SRFD_SECURITY_INTERNAL_H__

#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/hash.h>
#include "srfd_debug.h"
#include "srfd_security.h"


/* Hash table to store the SRFD structures
*/


#define SRFD_SUBGROUP_HASH_BITS 12
#define SRFD_MAX_SUBGROUP_HASH (1u << SRFD_SUBGROUP_HASH_BITS)

#define SRFD_CREDSEC_HASH_BITS 16
#define SRFD_MAX_CREDSEC_HASH (1u << SRFD_CREDSEC_HASH_BITS)

#define SRFD_SUBPID_HASH_BITS 12
#define SRFD_MAX_SUBPID_HASH (1u << SRFD_SUBPID_HASH_BITS)

#define SRFD_OBJECT_HASH_BITS 16
#define SRFD_MAX_OBJECT_HASH (1u << SRFD_OBJECT_HASH_BITS)

#define SRFD_HANDLE_HASH_BITS 12
#define SRFD_MAX_HANDLE_HASH (1u << SRFD_HANDLE_HASH_BITS)


extern struct list_head srfd_object_list[SRFD_MAX_OBJECT_HASH];
extern struct list_head srfd_subgroup_list[SRFD_MAX_SUBGROUP_HASH];
extern struct list_head srfd_credsec_list[SRFD_MAX_CREDSEC_HASH];
extern struct list_head srfd_subpid_list[SRFD_MAX_SUBPID_HASH];
extern struct list_head srfd_handle_list[SRFD_MAX_HANDLE_HASH];

extern atomic_t srfd_subgroup_activeCount;
extern atomic_t srfd_credsec_activeCount;
extern atomic_t srfd_subpid_activeCount;
extern atomic_t srfd_object_activeCount;
extern atomic_t srfd_handle_activeCount;

#endif /* __SRFD_SECURITY_INTERNAL_H__ */

