/* * Self-Revocation Free Downgrading - A kernel module that applies information flow
 * control to protect system integrity without self-revocation
 * Copyright (C) 2014 Secure Systems Laboratory, Stony Brook University
 *
 * This file is part of Self-Revocation Free Downgrading.
 *
 * Self-Revocation Free Downgrading is free software: you can redistribute it
 * and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Self-Revocation Free Downgrading is distributed in the hope that it will be
 * useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Self-Revocation Free Downgrading.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>

const char *HIGHI_FILE = "./testfiles/highI";
const char *LOWI_FILE = "./testfiles/lowI";

FILE *highI = NULL, *lowI = NULL;


#define PPI_LOGG(format, args...) printf(format "\n", ##args)

#define PPI_OK PPI_LOGG
#define PPI_ERROR PPI_LOGG
#define PPI_NOT_EXPECTED PPI_LOGG

#define PPI_LOG(_level, _format, _args...) PPI_ ##_level( "[" #_level "] " _format, ##_args)


#define PPI_EXPECTED(_x) do {\
		if (!_x) { \
			PPI_LOG(NOT_EXPECTED, "Assert failed: " #_x); \
			PPI_ERROR("Aborting current test: " currentTestName); \
			goto nextTest; \
		} \
	} while (0)



FILE *openHighI(char *_mode) {
	highI = fopen(HIGHI_FILE, _mode);
	return highI;
}

FILE *openLowI(char *_mode) {
	lowI = fopen(LOWI_FILE, _mode);
	return lowI;
}



void flushEnv() {
	if (highI) {
		fclose(highI);
		highI = NULL;
	}
	if (lowI) {
		fclose(lowI);
		lowI = NULL;
	}
}



void main() {


	/*
	Test 1
		With high integrity file opened in read-write mode, low integrity file cannot be opened in read-write mode
	*/

#define nextTest test2
#define currentTestName "Test 1: Cannot open lowI while highI is opened in WRITE mode"
	PPI_EXPECTED(openHighI("r+"));
	PPI_EXPECTED(!openLowI("r+"));
	PPI_LOG(OK, currentTestName);

test2:

#undef nextTest
#define nextTest test3
#undef currentTestName
#define currentTestName "Test 2: Can open lowI while highI is opened in READ mode"
	/* Test 2:
		With high integrity file opened in read mode, low integrity file can be opened
	*/
	flushEnv();
	PPI_EXPECTED(openHighI("r"));
	PPI_EXPECTED(openLowI("r+"));
	PPI_LOG(OK, currentTestName);

test3:
	flushEnv();	

}
